/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 *  Copyright (C) 1996  Ralf Baechle
 */
#include <linux/sched.h>

#include <asm/cache.h>
#include <asm/mipsregs.h>
#include <asm/processor.h>

extern asmlinkage void mips2_cacheflush(void *addr, int nbytes, unsigned int flags);

void (*mips_cache_init)(void);

static void
mips2_cache_init(void)
{
	cacheflush = mips2_cacheflush;
}

void (*switch_to_user_mode)(struct pt_regs *regs);

static void
mips2_switch_to_user_mode(struct pt_regs *regs)
{
	regs->cp0_status = regs->cp0_status | ST0_KUC;
}

unsigned long (*thread_saved_pc)(struct thread_struct *t);

/*
 * Return saved PC of a blocked thread.
 */
static unsigned long mips2_thread_saved_pc(struct thread_struct *t)
{
	return ((unsigned long *)(unsigned long)t->reg29)[13];
}

unsigned long (*get_wchan)(struct task_struct *p);

static unsigned long mips2_get_wchan(struct task_struct *p)
{
	/*
	 * This one depends on the frame size of schedule().  Do a
	 * "disass schedule" in gdb to find the frame size.  Also, the
	 * code assumes that sleep_on() follows immediately after
	 * interruptible_sleep_on() and that add_timer() follows
	 * immediately after interruptible_sleep().  Ugly, isn't it?
	 * Maybe adding a wchan field to task_struct would be better,
	 * after all...
	 */
	unsigned long schedule_frame;
	unsigned long pc;

	pc = thread_saved_pc(&p->tss);
	if (pc >= (unsigned long) interruptible_sleep_on && pc < (unsigned long) add_timer) {
		schedule_frame = ((unsigned long *)(long)p->tss.reg30)[13];
		return ((unsigned long *)schedule_frame)[11];
	}
	return pc;
}

void
mips2_cpu_init(void)
{
	mips_cache_init = mips2_cache_init;
	switch_to_user_mode = mips2_switch_to_user_mode;
	thread_saved_pc = mips2_thread_saved_pc;
	get_wchan = mips2_get_wchan;
}
