/*
 * include/asm-mips/segment.h
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994, 1995 by Ralf Baechle
 *
 * Note that the quad functions are only being used for the 64 bit kernel and
 * therefore it isn't really important that they will be miscompiled for
 * 32-bit kernels.
 */
#ifndef __ASM_MIPS_SEGMENT_H
#define __ASM_MIPS_SEGMENT_H

#ifndef __LANGUAGE_ASSEMBLY__
/*
 * For memcpy()
 */
#include <linux/string.h>

/*
 * Uh, these should become the main single-value transfer routines..
 * They automatically use the right size if we just have the right
 * pointer type..
 * As MIPS uses the same address space for kernel and user data, we
 * can just do these as direct assignments.
 */
#define put_user(x,ptr)        do { (*(ptr)=(x)); } while (0)
#define get_user(ptr)  (*(ptr))

/*
 * These are deprecated..
 *
 * Use "put_user()" and "get_user()" with the proper pointer types instead.
 */
#define get_fs_byte(addr) get_user((const unsigned char *)(addr))
#define get_fs_word(addr) get_user((const unsigned short *)(addr))
#define get_fs_long(addr) get_user((const unsigned int *)(addr))
#define get_fs_quad(addr) get_user((const unsigned long long *)(addr))

#define put_fs_byte(x,addr) put_user((x),(unsigned char *)(addr))
#define put_fs_word(x,addr) put_user((x),(unsigned short *)(addr))
#define put_fs_long(x,addr) put_user((x),(unsigned int *)(addr))
#define put_fs_quad(x,addr) put_user((x),(unsigned long long *)(addr))

#define memcpy_fromfs(to,from,n) memcpy((to),(from),(n))
#define memcpy_tofs(to,from,n) memcpy((to),(from),(n))

/*
 * The fs value determines whether argument validity checking should be
 * performed or not.  If get_fs() == USER_DS, checking is performed, with
 * get_fs() == KERNEL_DS, checking is bypassed.
 *
 * For historical reasons, these macros are grossly misnamed.
 */
#define KERNEL_DS 0
#define USER_DS 1

#define get_fs()        (current->tss.segment)
#define set_fs(x)       (current->tss.segment=(x))

static inline unsigned long get_ds(void)
{
	return KERNEL_DS;
}

#endif /* !__LANGUAGE_ASSEMBLY__ */

/*
 * Memory segments (32bit kernel mode addresses)
 */
#define KUSEG                   0x00000000
#define KSEG0                   0x80000000
#define KSEG1                   0xa0000000
#define KSEG2                   0xc0000000
#define KSEG3                   0xe0000000

/*
 * Returns the kernel segment base of a given address
 */
#define KSEGX(a)                (((unsigned long)(a)) & 0xe0000000)

/*
 * Returns the physical address of a KSEG0/KSEG1 address
 */
#define PHYSADDR(a)		(((unsigned long)(a)) & 0x1fffffff)

/*
 * Map an address to a certain kernel segment
 */
#define KSEG0ADDR(a)		((__typeof__(a))(((unsigned long)(a) & 0x1fffffff) | KSEG0))
#define KSEG1ADDR(a)		((__typeof__(a))(((unsigned long)(a) & 0x1fffffff) | KSEG1))
#define KSEG2ADDR(a)		((__typeof__(a))(((unsigned long)(a) & 0x1fffffff) | KSEG2))
#define KSEG3ADDR(a)		((__typeof__(a))(((unsigned long)(a) & 0x1fffffff) | KSEG3))

/*
 * Memory segments (64bit kernel mode addresses)
 */
#define XKUSEG                  0x0000 0000 0000 0000
#define XKSSEG                  0x4000 0000 0000 0000
#define XKPHYS                  0x8000 0000 0000 0000
#define XKSEG                   0xc000 0000 0000 0000
#define CKSEG0                  0xffff ffff 8000 0000
#define CKSEG1                  0xffff ffff a000 0000
#define CKSSEG                  0xffff ffff c000 0000
#define CKSEG3                  0xffff ffff e000 0000

#endif /* __ASM_MIPS_SEGMENT_H */
